<?php

declare(strict_types=1);

namespace Dedi\SyliusOrderStatusHistoryPlugin\Workflow;

use Dedi\Contracts\SyliusPluginSettings\SettingsProviderInterface;
use Dedi\SyliusOrderStatusHistoryPlugin\Entity\OrderStatusHistoryInterface;
use Doctrine\ORM\EntityManagerInterface;
use Psr\Log\LoggerInterface;
use SM\Event\TransitionEvent;
use Sylius\Component\Core\Model\OrderInterface;
use Sylius\Component\Core\OrderPaymentTransitions;
use Sylius\Component\Core\OrderShippingTransitions;
use Sylius\Component\Order\OrderTransitions;
use Sylius\Component\Resource\Factory\FactoryInterface;
use Symfony\Component\HttpFoundation\RequestStack;

class OrderWorkflowCallback
{
    /** @param FactoryInterface<OrderStatusHistoryInterface> $orderStatusHistoryFactory */
    public function __construct(
        private SettingsProviderInterface $settingsProvider,
        private FactoryInterface $orderStatusHistoryFactory,
        private EntityManagerInterface $entityManager,
        private RequestStack $requestStack,
        private LoggerInterface $logger,
    ) {
    }

    public function apply(OrderInterface $order, TransitionEvent $event): void
    {
        try {
            $settings = $this->settingsProvider->provide();

            if (!$settings->isEnabled()) {
                return;
            }

            $request = $this->requestStack->getCurrentRequest();

            $stateMachineName = $event->getStateMachine()->getGraph();

            /** @var OrderStatusHistoryInterface $orderStatusHistory */
            $orderStatusHistory = $this->orderStatusHistoryFactory->createNew();
            $orderStatusHistory->setOrder($order);
            $orderStatusHistory->setStateMachineName($stateMachineName);
            $orderStatusHistory->setPreviousState($event->getState());
            $orderStatusHistory->setTransition($event->getTransition());
            $orderStatusHistory->setUrl($request?->getUri());

            switch ($stateMachineName) {
                case OrderTransitions::GRAPH:
                    $orderStatusHistory->setState($order->getState());

                    break;
                case OrderPaymentTransitions::GRAPH:
                    $orderStatusHistory->setPaymentState($order->getPaymentState());

                    break;
                case OrderShippingTransitions::GRAPH:
                    $orderStatusHistory->setShippingState($order->getShippingState());

                    break;
            }

            $this->entityManager->persist($orderStatusHistory);
        } catch (\Throwable $exception) {
            $this->logger->error($exception->getMessage(), ['exception' => $exception]);
        }
    }
}
